/*
    Copyright (c) 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifdef HAVE_CONFIG_H
#include "autoconfig.h"
#endif

#include <stdio.h>
#include <time.h>
#include <sstream>

#include "html_web_handler.h"

#define PAGE_START "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\
<html>"
#define PAGE_END "</html>"

#define CSS "<style type=\"text/css\">\
 *.report {font-family: arial, helvetica, tahoma, sans-serif;font-size: 12px;}\
 h1.report {padding:4px;color:#005138;font-size: xx-large !important;}\
 h2.report {padding:4px; color:#005138; font-size: x-large !important;}\
 h3.report {padding:4px; color:#005138; font-size: large !important;}\
 p.report {color:#005138;padding:4px 6px; font-size: 12px;}\
 div.report {padding:4px 6px;}\
 td.report {vertical-align: baseline;}\
 th.report {vertical-align: top; background-color: #CCCCCC;}\
 table.report {text-align: left; width: 100%;}\
.text-link a:link {text-decoration: underline; color:#005138;}\
.text-link a:visited {text-decoration: underline; color:#005138;}\
.green-text {font-size: 12px; color:#005138;}\
.green-text-big {font-size: 16px; color:#005138;}\
.green-text-big .black-text-big {color:#000000;}\
.green-text .black-text {color:#000000;}\
.red-text {font-size: 12px; color:#FF0000;}\
</style>"

#define HEAD_START "<head>\
<meta content=\"text/html; charset=UTF-8\" http-equiv=\"Content-Type\"><title>"

#define HEAD_END "</title>"CSS"</head>"

#define BODY_START "<body class=\"report\"><h1 class=\"report\">"

#define BODY_DESC "</h1><h2 class=\"report\">"
#define BODY "</h2><div class=\"report black-text-big\">"
#define BODY_END "</div></body>"

#define DIV_CLASS_START "<div class=\"report "
#define DIV_CLASS_END "\">"
#define DIV_TAG_END "</div>"

#define SPAN_CLASS_START "<span class=\"report "
#define SPAN_CLASS_END "\">"
#define SPAN_TAG_END "</span>"

#define H3_TAG_START "<h3 class=\"report\">"
#define H3_TAG_END "</h3>"

#define BREAK "<br/>"

#define TABLE_START "<table class=\"report\""
#define TABLE_NO_BORDER " border=\"0\""
#define TABLE_BORDER " border=\"1\""
#define TABLE_START_CONT " cellpadding=\"2\" cellspacing=\"2\">"\
                         "<tbody class=\"report\">"
#define TABLE_END "</tbody></table>"
#define TABLE_ROW_START "<tr class=\"report\">"
#define TABLE_ROW_END "</tr>"
#define TABLE_COLUMN_START_NOSPAN_NOWIDTH "<td class=\"report\">"
#define TABLE_COLUMN_TAG_START "<td class=\"report\""
#define TABLE_COLUMN_TAG_END ">"
#define TABLE_COLUMN_SPAN_START " colspan=\""
#define TABLE_COLUMN_SPAN_END "\""
#define TABLE_COLUMN_WIDTH_START " width=\""
#define TABLE_COLUMN_WIDTH_END "\""
#define TABLE_COLUMN_END "</td>"
#define TABLE_HEADER_TAG_START "<th class=\"report\">"
#define TABLE_HEADER_TAG_END "</th>"
#define TABLE_CAPTION_TAG_START "<caption class=\"report\">"
#define TABLE_CAPTION_TAG_END "</caption>"

namespace dSA
{

HTMLWebHandler::HTMLWebHandler(struct mg_connection* connection,
                               const struct mg_request_info* request_info) :
                               WebHandler(connection, request_info) {}

void HTMLWebHandler::failure(const std::string title, const std::string message)
{
    std::ostringstream result;
    result << webutil::render_http_header(500, "text/html; charset=UTF-8");
    result << PAGE_START << HEAD_START << title << HEAD_END;
    result << BODY_START << "digitalSTROM" << BODY_DESC << title;
    result << BODY << message << BODY_END << PAGE_END;
    mg_write(m_connection, result.str().c_str(),
             result.str().length());
}

std::string HTMLWebHandler::page_begin(const std::string title, bool download)
{
    std::ostringstream result;
    std::string attachment;
    if (download)
    {
        attachment = title + "_" + util::str(time(NULL)) + ".html";
    }
    result << webutil::render_http_header(200, "text/html; charset=UTF-8",
                                          attachment);
    result << PAGE_START << HEAD_START << title << HEAD_END;
    result << BODY_START << "digitalSTROM" << BODY_DESC << title;
    result << BODY;
    return result.str();
}

std::string HTMLWebHandler::page_end()
{
    std::ostringstream result;
    result << BODY_END << PAGE_END;
    return result.str();
}

std::string HTMLWebHandler::table_begin(bool border)
{
    std::ostringstream result;
    result << TABLE_START;
    result << (border ? TABLE_BORDER : TABLE_NO_BORDER);
    result << TABLE_START_CONT;
    return result.str();
}

std::string HTMLWebHandler::table_end()
{
    return TABLE_END;
}

std::string HTMLWebHandler::table_row_begin()
{
    return TABLE_ROW_START;
}

std::string HTMLWebHandler::table_row_end()
{
    return TABLE_ROW_END;
}

std::string HTMLWebHandler::table_column_add(const std::string text,
                                             const unsigned short colspan,
                                             const unsigned short width)
{
    std::ostringstream result;
    if ((colspan == 0) && (width == 0))
    {
        result << TABLE_COLUMN_START_NOSPAN_NOWIDTH;
    }
    else
    {
        result << TABLE_COLUMN_TAG_START;

        if (colspan)
        {
            result << TABLE_COLUMN_SPAN_START << util::str(colspan);
            result << TABLE_COLUMN_SPAN_END;
        }

        if (width)
        {
            result << TABLE_COLUMN_WIDTH_START << util::str(width);
            result << TABLE_COLUMN_WIDTH_END;
        }

        result << TABLE_COLUMN_TAG_END;
    }

    result << text << TABLE_COLUMN_END;
    return result.str();
}

std::string HTMLWebHandler::error(const std::string text)
{
    std::ostringstream result;
    result << SPAN_CLASS_START << "red-text" << SPAN_CLASS_END;
    result << BREAK << text << SPAN_TAG_END;
    return result.str();
}

std::string HTMLWebHandler::header(const std::string text)
{
    std::ostringstream result;
    result << H3_TAG_START << text << H3_TAG_END;
    return result.str();
}

std::string HTMLWebHandler::table_row_simple(const std::string c1,
                                             const std::string c2)
{
    std::ostringstream result;
    result << table_row_begin() << table_column_add(c1);
    result << table_column_add(c2) << table_row_end();
    return result.str();
}

std::string HTMLWebHandler::table_header(const std::string text)
{
    std::ostringstream result;
    result << TABLE_HEADER_TAG_START << text << TABLE_HEADER_TAG_END;
    return result.str();
}

std::string HTMLWebHandler::table_caption(const std::string text)
{
    std::ostringstream result;
    result << TABLE_CAPTION_TAG_START << text << TABLE_CAPTION_TAG_END;
    return result.str();
}

} // namespace

