/*
    Copyright (c) 2014 digitalSTROM AG, Zurich, Switzerland
    
    Imported from the digitalSTROM Server project.

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of digitalSTROM Assistant.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifdef HAVE_CONFIG_H
#include "autoconfig.h"
#endif

#include <iostream>

#include "vdc-connection.h"
#include "messages/vdc-messages.pb.h"
#include "stringconverter.h"
#include "exceptions.h"
#include "ds-assistant.h"
#include "util.h"

namespace dSA {

std::string VdcHelper::getHardwareGuid(dsuid_t _vdsm, dsuid_t _device) {
    vdcapi::Message message;

    std::string ret;

    message.set_type(vdcapi::VDSM_REQUEST_GET_PROPERTY);
    vdcapi::vdsm_RequestGetProperty *getprop =
        message.mutable_vdsm_request_get_property();
    getprop->set_dsuid(util::str(_device));

    vdcapi::PropertyElement *query = getprop->add_query();
    query->set_name("hardwareGuid");

    uint8_t buffer_in[4096];
    uint8_t buffer_out[4096];
    uint16_t bs;

    memset(buffer_in, 0, sizeof(buffer_in));
    memset(buffer_out, 0, sizeof(buffer_out));

    if (!message.SerializeToArray(buffer_in, sizeof(buffer_in))) {
        throw _Exception("could not serialize message");
    }

    dSA::dS_Assistant::getInstance()->dS485()->protobufMessageRequest(
            _vdsm, message.ByteSize(), buffer_in, &bs, buffer_out);

    message.Clear();
    if (bs > sizeof(buffer_out)) {
        throw _Exception("incoming message too large, dropping");
    }

    if (!message.ParseFromArray(buffer_out, bs)) {
        throw _Exception("could not parse response message");
    }

    // error message
    if (message.type() == vdcapi::GENERIC_RESPONSE) {
        throw _Exception("received error with code " +
                util::str(message.generic_response().code()));
    }
    if (!message.has_vdc_response_get_property()) {
        throw _Exception("received unexpected reply");
    }

    vdcapi::vdc_ResponseGetProperty response =
        message.vdc_response_get_property();

    for (int i = 0; i < response.properties_size(); i++) {
        vdcapi::PropertyElement el = response.properties(i);

        if (!el.has_name()) {
            continue;
        }

        vdcapi::PropertyValue val = el.value();

        if (!val.has_v_string()) {
            continue;
        }

        StringConverter st("UTF-8", "UTF-8");
        if (el.name() == "hardwareGuid") {
            try {
                ret = st.convert(val.v_string());
            } catch (Exception& e) {}
            break;
        }
    }

    return ret;
}

std::string VdcHelper::getHardwareInfo(dsuid_t _vdsm, dsuid_t _device) {
    vdcapi::Message message;

    std::string ret;

    message.set_type(vdcapi::VDSM_REQUEST_GET_PROPERTY);
    vdcapi::vdsm_RequestGetProperty *getprop =
        message.mutable_vdsm_request_get_property();
    getprop->set_dsuid(util::str(_device));

    vdcapi::PropertyElement *query = getprop->add_query();
    query->set_name("model");

    uint8_t buffer_in[4096];
    uint8_t buffer_out[4096];
    uint16_t bs;

    memset(buffer_in, 0, sizeof(buffer_in));
    memset(buffer_out, 0, sizeof(buffer_out));

    if (!message.SerializeToArray(buffer_in, sizeof(buffer_in))) {
        throw _Exception("could not serialize message");
    }

    dSA::dS_Assistant::getInstance()->dS485()->protobufMessageRequest(
            _vdsm, message.ByteSize(), buffer_in, &bs, buffer_out);

    message.Clear();
    if (bs > sizeof(buffer_out)) {
        throw _Exception("incoming message too large, dropping");
    }

    if (!message.ParseFromArray(buffer_out, bs)) {
        throw _Exception("could not parse response message");
    }

    // error message
    if (message.type() == vdcapi::GENERIC_RESPONSE) {
        throw _Exception("received error with code " +
                         util::str(message.generic_response().code()));
    }
    if (!message.has_vdc_response_get_property()) {
        throw _Exception("received unexpected reply");
    }

    vdcapi::vdc_ResponseGetProperty response =
        message.vdc_response_get_property();

    for (int i = 0; i < response.properties_size(); i++) {
        vdcapi::PropertyElement el = response.properties(i);

        if (!el.has_name()) {
            continue;
        }

        vdcapi::PropertyValue val = el.value();

        if (!val.has_v_string()) {
            continue;
        }

        StringConverter st("UTF-8", "UTF-8");
        if (el.name() == "model") {
            try {
                ret = st.convert(val.v_string());
            } catch (Exception &ex) {}
            break;
        }
    }

    return ret;
}

}// namespace
